@@ -1,6 +1,6 @@
 // Copyright 2009, 2010 Hans Pirnay
 // All Rights Reserved.
-// This code is published under the Common Public License.
+// This code is published under the Eclipse Public License.
 //
 // Date   : 2009-05-06
 
@@ -33,7 +33,7 @@ namespace Ipopt
     /** Virtual functions overloaded from PCalculator */
     virtual bool ComputeP();
 
-    virtual bool GetSchurMatrix(const SchurData& B, Matrix& S);
+    virtual bool GetSchurMatrix(const SmartPtr<const SchurData>& B, SmartPtr<Matrix>& S);
 
     virtual void PrintImpl(const Journalist& jnlst,
 			   EJournalLevel level,
