@@ -1,8 +1,8 @@
 // Copyright (C) 2009 International Business Machines and others.
 // All Rights Reserved.
-// This code is published under the Common Public License.
+// This code is published under the Eclipse Public License.
 //
-// $Id: IpIterativeWsmpSolverInterface.cpp 1692 2010-06-21 19:10:07Z andreasw $
+// $Id: IpIterativeWsmpSolverInterface.cpp 1864 2010-12-22 19:21:02Z andreasw $
 //
 // Authors:  Andreas Waechter                 IBM    2009-09-18
 //               based on IpWsmpSolverInterface.cpp (rev 1521)
@@ -21,6 +21,16 @@
 # endif
 #endif
 
+#ifdef HAVE_CSTDIO
+# include <cstdio>
+#else
+# ifdef HAVE_STDIO_H
+#  include <stdio.h>
+# else
+#  error "don't have header file for stdio"
+# endif
+#endif
+
 /** Prototypes for WISMP's subroutines */
 extern "C"
 {
