@@ -1,14 +1,15 @@
 // Copyright (C) 2005, 2010 International Business Machines and others.
 // All Rights Reserved.
-// This code is published under the Common Public License.
+// This code is published under the Eclipse Public License.
 //
-// $Id: IpPardisoSolverInterface.cpp 1692 2010-06-21 19:10:07Z andreasw $
+// $Id: IpPardisoSolverInterface.cpp 1864 2010-12-22 19:21:02Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2005-03-17
 //
 //           Olaf Schenk                      Univ of Basel 2005-09-20
 //                  - changed options, added PHASE_ flag
 
+
 #include "IpoptConfig.h"
 #include "IpPardisoSolverInterface.hpp"
 # include <math.h>
@@ -44,7 +45,7 @@
 #endif
 
 // determine the correct name of the Pardiso function
-#ifdef _MSC_VER
+#if defined(_MSC_VER) && defined(HAVE_PARDISO)
 # define PARDISOINIT_FUNC PARDISOINIT
 # define PARDISO_FUNC PARDISO
 #else
@@ -52,6 +53,7 @@
 # define PARDISO_FUNC F77_FUNC(pardiso,PARDISO)
 #endif
 
+
 /* Prototypes for Pardiso's subroutines */
 extern "C"
 {
@@ -73,6 +75,13 @@ extern "C"
                     const ipfint* NRHS, ipfint* IPARM,
                     const ipfint* MSGLVL, double* B, double* X,
                     ipfint* ERROR, double* DPARM);
+
+
+#ifdef PARDISO_MATCHING_PREPROCESS
+  void smat_reordering_pardiso_wsmp_(const ipfint* N, const ipfint* ia, const ipfint* ja, const double* a_, ipfint* a2, ipfint* ja2,  double* a2_,
+                                     ipfint* perm2,  double* scale2, ipfint* tmp2_, ipfint preprocess );
+#endif
+
 }
 
 namespace Ipopt
@@ -84,6 +93,15 @@ namespace Ipopt
   PardisoSolverInterface::PardisoSolverInterface()
       :
       a_(NULL),
+
+#ifdef PARDISO_MATCHING_PREPROCESS
+      ia2(NULL),
+      ja2(NULL),
+      a2_(NULL),
+      perm2(NULL),
+      scale2(NULL),
+#endif
+
       negevals_(-1),
       initialized_(false),
 
@@ -123,6 +141,15 @@ namespace Ipopt
     delete[] IPARM_;
     delete[] DPARM_;
     delete[] a_;
+
+#ifdef PARDISO_MATCHING_PREPROCESS
+    delete[] ia2;
+    delete[] ja2;
+    delete[] a2_;
+    delete[] perm2;
+    delete[] scale2;
+#endif
+
   }
 
   void PardisoSolverInterface::RegisterOptions(SmartPtr<RegisteredOptions> roptions)
@@ -167,7 +194,7 @@ namespace Ipopt
       "This is MSGLVL in the Pardiso manual.");
     roptions->AddStringOption2(
       "pardiso_skip_inertia_check",
-      "Always pretent inertia is correct.",
+      "Always pretend inertia is correct.",
       "no",
       "no", "check inertia",
       "yes", "skip inertia check",
@@ -298,6 +325,23 @@ namespace Ipopt
     delete[] a_;
     a_ = NULL;
 
+#ifdef PARDISO_MATCHING_PREPROCESS
+    delete[] ia2;
+    ia2 = NULL;
+
+    delete[] ja2;
+    ja2 = NULL;
+
+    delete[] a2_;
+    a2_ = NULL;
+
+    delete[] perm2;
+    perm2 = NULL;
+
+    delete[] scale2;
+    scale2 = NULL;
+#endif
+
     // Call Pardiso's initialization routine
     IPARM_[0] = 0;  // Tell it to fill IPARM with default values(?)
 
@@ -399,6 +443,7 @@ namespace Ipopt
       // e.g.  pardiso_inverse_norm_factor
       // Default is 500
       // 2 <= value < 50000
+      DPARM_[ 8] = 25; // maximum number of non-improvement steps
     }
 
     MSGLVL_ = pardiso_msglvl;
@@ -534,6 +579,33 @@ namespace Ipopt
 
       fclose (mat_file);
     }
+    /* addtional matrix format */
+    if (getenv ("IPOPT_WRITE_MAT_MTX")) {
+      /* Write header */
+      FILE    *mat_file;
+      char     mat_name[128];
+      char     mat_pref[32];
+
+      ipfint   i;
+      ipfint   j;
+
+      if (getenv ("IPOPT_WRITE_PREFIX"))
+        strcpy (mat_pref, getenv ("IPOPT_WRITE_PREFIX"));
+      else
+        strcpy (mat_pref, "mat-ipopt");
+
+      Snprintf (mat_name, 127, "%s_%03d-%02d.mtx",
+                mat_pref, iter_cnt, sol_cnt);
+
+      // Open and write matrix file.
+      mat_file = fopen (mat_name, "w");
+
+      for (i = 0; i < N; i++)
+        for (j = ia[i]; j < ia[i+1]-1; j++)
+          fprintf (mat_file, " %d %d %32.24e \n", i+1, ja[j-1], a_[j-1]);
+
+      fclose (mat_file);
+    }
   }
 
   ESymSolverStatus
@@ -564,10 +636,44 @@ namespace Ipopt
           IpData().TimingStats().LinearSystemSymbolicFactorization().Start();
         }
         PHASE = 11;
+
+#ifdef PARDISO_MATCHING_PREPROCESS
+        delete[] ia2;
+        ia2 = NULL;
+
+        delete[] ja2;
+        ja2 = NULL;
+
+        delete[] a2_;
+        a2_ = NULL;
+
+        delete[] perm2;
+        perm2 = NULL;
+
+        delete[] scale2;
+        scale2 = NULL;
+
+        ia2    = new ipfint[N+1];
+        ja2    = new ipfint[nonzeros_];
+        a2_    = new double[nonzeros_];
+        perm2  = new ipfint[N];
+        scale2 = new double[N];
+        ipfint* tmp2_  = new ipfint[N];
+
+        smat_reordering_pardiso_wsmp_(&N, ia, ja, a_, ia2, ja2, a2_, perm2, scale2, tmp2_, 0);
+
+        delete[] tmp2_;
+
+#endif
+
         Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
                        "Calling Pardiso for symbolic factorization.\n");
         PARDISO_FUNC(PT_, &MAXFCT_, &MNUM_, &MTYPE_,
+#ifdef PARDISO_MATCHING_PREPROCESS
+                     &PHASE, &N, a2_, ia2, ja2, &PERM,
+#else
                      &PHASE, &N, a_, ia, ja, &PERM,
+#endif
                      &NRHS, IPARM_, &MSGLVL_, &B, &X,
                      &ERROR, DPARM_);
         if (HaveIpData()) {
@@ -613,8 +719,20 @@ namespace Ipopt
         debug_last_iter_ = 0;
       }
 
+#ifdef PARDISO_MATCHING_PREPROCESS
+      {
+        ipfint* tmp2_  = new ipfint[N];
+        smat_reordering_pardiso_wsmp_ (&N, ia, ja, a_, ia2, ja2, a2_, perm2, scale2, tmp2_, 1);
+        delete[] tmp2_;
+      }
+#endif
+
       PARDISO_FUNC(PT_, &MAXFCT_, &MNUM_, &MTYPE_,
+#ifdef PARDISO_MATCHING_PREPROCESS
+                   &PHASE, &N, a2_, ia2, ja2, &PERM,
+#else
                    &PHASE, &N, a_, ia, ja, &PERM,
+#endif
                    &NRHS, IPARM_, &MSGLVL_, &B, &X,
                    &ERROR, DPARM_);
       if (HaveIpData()) {
@@ -710,6 +828,7 @@ namespace Ipopt
     ipfint PERM;   // This should not be accessed by Pardiso
     ipfint NRHS = nrhs;
     double* X = new double[nrhs*dim_];
+
     double* ORIG_RHS = new double[nrhs*dim_];
     ipfint ERROR;
     // Initialize solution with zero and save right hand side
@@ -723,7 +842,12 @@ namespace Ipopt
     if (HaveIpData()) {
       iter_count = IpData().iter_count();
     }
-    write_iajaa_matrix (N, ia, ja, a_, rhs_vals, iter_count, debug_cnt_);
+
+#ifdef PARDISO_MATCHING_PREPROCESS
+    write_iajaa_matrix (N, ia2, ja2, a2_, rhs_vals, iter_count, debug_cnt_);
+#else
+    write_iajaa_matrix (N,  ia,  ja,  a_, rhs_vals, iter_count, debug_cnt_);
+#endif
 
     int attempts = 0;
     const int max_attempts =
@@ -731,6 +855,23 @@ namespace Ipopt
 
     while (attempts < max_attempts) {
 
+
+#ifdef PARDISO_MATCHING_PREPROCESS
+      for (int i = 0; i < N; i++) {
+        rhs_vals[perm2[i]] = scale2[i] * ORIG_RHS[ i  ];
+      }
+      PARDISO_FUNC(PT_, &MAXFCT_, &MNUM_, &MTYPE_,
+                   &PHASE, &N, a2_, ia2, ja2, &PERM,
+                   &NRHS, IPARM_, &MSGLVL_, rhs_vals, X,
+                   &ERROR, DPARM_);
+      for (int i = 0; i < N; i++) {
+        X[i] = rhs_vals[ perm2[i]];
+      }
+      for (int i = 0; i < N; i++) {
+        rhs_vals[i] =  scale2[i]*X[i];
+      }
+
+#else
       for (int i = 0; i < N; i++) {
         rhs_vals[i] = ORIG_RHS[i];
       }
@@ -738,6 +879,8 @@ namespace Ipopt
                    &PHASE, &N, a_, ia, ja, &PERM,
                    &NRHS, IPARM_, &MSGLVL_, rhs_vals, X,
                    &ERROR, DPARM_);
+#endif
+
 
       if (ERROR <= -100 && ERROR >= -102) {
         Jnlst().Printf(J_WARNING, J_LINEAR_ALGEBRA,
