@@ -1,8 +1,8 @@
-// Copyright (C) 2004, 2008 International Business Machines and others.
+// Copyright (C) 2004, 2009 International Business Machines and others.
 // All Rights Reserved.
-// This code is published under the Common Public License.
+// This code is published under the Eclipse Public License.
 //
-// $Id: IpTripletHelper.cpp 1492 2009-07-07 02:11:04Z andreasw $
+// $Id: IpTripletHelper.cpp 1864 2010-12-22 19:21:02Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-08-13
 
@@ -21,6 +21,7 @@
 #include "IpCompoundMatrix.hpp"
 #include "IpCompoundSymMatrix.hpp"
 #include "IpTransposeMatrix.hpp"
+#include "IpExpandedMultiVectorMatrix.hpp"
 
 #include "IpDenseVector.hpp"
 #include "IpCompoundVector.hpp"
@@ -98,6 +99,11 @@ namespace Ipopt
       return GetNumberEntries_(*trans);
     }
 
+    const ExpandedMultiVectorMatrix* expmv = dynamic_cast<const ExpandedMultiVectorMatrix*>(mptr);
+    if (expmv) {
+      return GetNumberEntries_(*expmv);
+    }
+
     THROW_EXCEPTION(UNKNOWN_MATRIX_TYPE,"Unknown matrix type passed to TripletHelper::GetNumberEntries");
   }
 
@@ -182,6 +188,12 @@ namespace Ipopt
       return;
     }
 
+    const ExpandedMultiVectorMatrix* expmv = dynamic_cast<const ExpandedMultiVectorMatrix*>(mptr);
+    if (expmv) {
+      FillRowCol_(n_entries, *expmv, row_offset, col_offset, iRow, jCol);
+      return;
+    }
+
     THROW_EXCEPTION(UNKNOWN_MATRIX_TYPE,"Unknown matrix type passed to TripletHelper::FillRowCol");
   }
 
@@ -266,6 +278,12 @@ namespace Ipopt
       return;
     }
 
+    const ExpandedMultiVectorMatrix* expmv = dynamic_cast<const ExpandedMultiVectorMatrix*>(mptr);
+    if (expmv) {
+      FillValues_(n_entries, *expmv, values);
+      return;
+    }
+
     THROW_EXCEPTION(UNKNOWN_MATRIX_TYPE,"Unknown matrix type passed to TripletHelper::FillValues");
   }
 
@@ -332,6 +350,13 @@ namespace Ipopt
     return GetNumberEntries(*matrix.OrigMatrix());
   }
 
+  Index TripletHelper::GetNumberEntries_(const ExpandedMultiVectorMatrix& matrix)
+  {
+    Index nRows = matrix.NRows();
+    Index dimVec = matrix.ExpandedMultiVectorMatrixOwnerSpace()->RowVectorSpace()->Dim();
+    return nRows*dimVec;
+  }
+
   void TripletHelper::FillRowCol_(Index n_entries, const GenTMatrix& matrix, Index row_offset, Index col_offset, Index* iRow, Index* jCol)
   {
     DBG_ASSERT(n_entries == matrix.Nonzeros());
@@ -739,6 +764,77 @@ namespace Ipopt
     FillValues(n_entries, *matrix.OrigMatrix(), values);
   }
 
+  void TripletHelper::FillRowCol_(Index n_entries, const ExpandedMultiVectorMatrix& matrix, Index row_offset, Index col_offset, Index* iRow, Index* jCol)
+  {
+    row_offset++;
+    col_offset++;
+    const Index nRows = matrix.NRows();
+    SmartPtr<const ExpansionMatrix> P = matrix.GetExpansionMatrix();
+    if (IsValid(P)) {
+      const Index* exppos = P->ExpandedPosIndices();
+      const Index nExp = P->NCols();
+      DBG_ASSERT(n_entries = nRows*nExp);
+      for (Index irow = row_offset; irow < row_offset+nRows; irow++) {
+        for (Index j=0; j<nExp; j++) {
+          *(iRow++) = irow;
+          *(jCol++) = col_offset + exppos[j];
+        }
+      }
+    }
+    else {
+      const Index nCols =  matrix.NCols();
+      DBG_ASSERT(n_entries = nRows*nCols);
+      for (Index irow = row_offset; irow < row_offset+nRows; irow++) {
+        for (Index jcol = col_offset; jcol < col_offset+nCols; jcol++) {
+          *(iRow++) = irow;
+          *(jCol++) = jcol;
+        }
+      }
+    }
+  }
+
+  void TripletHelper::FillValues_(Index n_entries, const ExpandedMultiVectorMatrix& matrix, Number* values)
+  {
+    const Index nRows = matrix.NRows();
+    SmartPtr<const ExpansionMatrix> P = matrix.GetExpansionMatrix();
+    if (IsValid(P)) {
+      const Index nExp = P->NCols();
+      double* vecvals = new double[nExp];
+      for (Index i=0; i<nRows; i++) {
+        SmartPtr<const Vector> vec = matrix.GetVector(i);
+        if (IsValid(vec)) {
+          DBG_ASSERT(vec->Dim() == nExp);
+          FillValuesFromVector(nExp, *vec, vecvals);
+          for (Index j=0; j<nExp; j++) {
+            *(values++) = vecvals[j];
+          }
+        }
+        else {
+          for (Index j=0; j<nExp; j++) {
+            *(values++) = 0.;
+          }
+        }
+      }
+      delete [] vecvals;
+    }
+    else {
+      const Index nCols =  matrix.NCols();
+      for (Index i=0; i<nRows; i++) {
+        SmartPtr<const Vector> vec = matrix.GetVector(i);
+        if (IsValid(vec)) {
+          DBG_ASSERT(vec->Dim() == nCols);
+          FillValuesFromVector(nCols, *vec, values);
+          values += nCols;
+        }
+        else {
+          for (Index j=0; j<nCols; j++) {
+            *(values++) = 0.;
+          }
+        }
+      }
+    }
+  }
+
   void TripletHelper::PutValuesInVector(Index dim, const double* values, Vector& vector)
   {
     DBG_ASSERT(dim == vector.Dim());
